﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

using org.apache.pdfbox.pdmodel;
using org.apache.pdfbox.exceptions;
using org.apache.pdfbox.util;

using java.lang;

namespace org.apache.pdfbox.examples.util
{

    using InvalidPasswordException = org.apache.pdfbox.exceptions.InvalidPasswordException;


    using PDDocument = org.apache.pdfbox.pdmodel.PDDocument;
    using PDPage = org.apache.pdfbox.pdmodel.PDPage;
    using PDStream = org.apache.pdfbox.pdmodel.common.PDStream;
    using PDFTextStripper = org.apache.pdfbox.util.PDFTextStripper;
    using TextPosition = org.apache.pdfbox.util.TextPosition;
    using System.IO;



    /// <summary>
    /// This is an example on how to get some x/y coordinates of text.
    /// 
    /// Usage: java org.apache.pdfbox.examples.util.PrintWordLocations &lt;input-pdf&gt;
    /// 
    /// @author <a href="mailto:ben@benlitchfield.com">Ben Litchfield</a>
    /// @version $Revision: 1.7 $
    /// </summary>
    //public class PrintWordLocations : org.apache.pdfbox.util.PDFTextStripper
        public class PrintWordLocations : org.apache.pdfbox.util.PDFTextStripper
    {
        public class WordBox
        {
            private readonly PrintWordLocations outerInstance;

            public float _xmin;
            public float _ymin;
            public float _fontsize;
            public float _xscale;
            public float _yscale;
            public float _height;
            public float _width;

            public WordBox(PrintWordLocations outerInstance, TextPosition text)
            {
                this.outerInstance = outerInstance;
                _xmin = text.getXDirAdj();
                _ymin = text.getYDirAdj();
                _fontsize = text.getFontSize();
                _xscale = text.getXScale();
                _yscale = text.getYScale();
                _height = text.getHeightDir();
                _width = text.getWidthDirAdj();
            }

            public virtual bool rejects(TextPosition text)
            {
                return (text.getXDirAdj() < _xmin) || (text.getYDirAdj() + text.getWidthOfSpace() < _ymin);
            }

            public virtual bool accepts(TextPosition text)
            {
                return !rejects(text);
            }

            public virtual void extendBy(TextPosition text)
            {
                float current_xmin = _xmin;
                float current_xmax = _xmin + _width;
                float current_ymin = _ymin;
                float current_ymax = _ymin + _height;

                float text_xmin = text.getXDirAdj();
                float text_xmax = text_xmin + text.getWidthDirAdj();
                float text_ymin = text.getYDirAdj();
                float text_ymax = text_ymin + text.getHeightDir();

                float new_xmin = java.lang.Math.min(current_xmin, text_xmin);
                float new_xmax = java.lang.Math.max(current_xmax, text_xmax);
                float new_ymin = java.lang.Math.min(current_ymin, text_ymin);
                float new_ymax = java.lang.Math.max(current_ymax, text_ymax);

                _xmin = new_xmin;
                _width = new_xmax - new_xmin;
                _ymin = new_ymin;
                _height = new_ymax - new_ymin;
            }
        }

        protected internal java.lang.StringBuilder word = new java.lang.StringBuilder("");
        protected internal char? last_character = new char?('\0');
        protected internal LinkedList<WordBox> box_list = new LinkedList<WordBox>();

        /// <summary>
        /// Default constructor.
        /// </summary>
        /// <exception cref="IOException"> If there is an error loading text stripper properties. </exception>
        //JAVA TO C# CONVERTER WARNING: Method 'throws' clauses are not available in .NET:
        //ORIGINAL LINE: public PrintWordLocations() throws java.io.IOException
        public PrintWordLocations()
        {
            try
            {
                base.setSortByPosition(true);
            }
            catch (System.Exception ex)
            {
                Console.Error.WriteLine(ex.ToString());

            }
            //base.SortByPosition = true;

        }

        //JAVA TO C# CONVERTER WARNING: Method 'throws' clauses are not available in .NET:
        //ORIGINAL LINE: public void processDocuments(String[] args) throws Exception
        public virtual void processDocuments(string[] args)
        {
            if (args.Length != 0)
            {
                usage();
            }
            else
            {
                PDDocument document = null;
                string InputFilePath = @"D:\Projects\PDF2Alto\Sample Project\Pdf2Text\ConsoleApplication1\temp\fmp000000426_0541.pdf";
                string OutputFilePath = @"D:\Projects\PDF2Alto\Sample Project\Pdf2Text\ConsoleApplication1\temp\test.txt";
                TextWriter tsw = new StreamWriter(OutputFilePath,true);

                try
                {

                    //document = PDDocument.load(args[0]);
                    document = PDDocument.load(InputFilePath);
                                        

                    if (document.isEncrypted())
                    {
                        try
                        {
                            document.decrypt("");
                        }
                        catch (InvalidPasswordException e)
                        {
                            Console.Error.WriteLine("Error: Document is encrypted with a password.");
                            Environment.Exit(1);
                        }
                    }
                    PrintWordLocations printer = new PrintWordLocations();
                    //PrintTextLocations printer = new PrintTextLocations();
                    //IList allPages = document.getDocumentCatalog.AllPages;

                    IList allPages = document.getDocumentCatalog().getAllPages().toArray();

                    //Console.WriteLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?><alto xmlns=\"http://www.loc.gov/standards/alto/alto-v2.0.xsd\"><Description><MeasurementUnit>inch1200</MeasurementUnit></Description><Layout>");
                    tsw.WriteLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?><alto xmlns=\"http://www.loc.gov/standards/alto/alto-v2.0.xsd\"><Description><MeasurementUnit>inch1200</MeasurementUnit></Description><Layout>");
                    

                    for (int i = 0; i < allPages.Count; i++)
                    {
                        PDPage page = (PDPage)allPages[i];
                        
                        //Console.WriteLine("<Page>");
                        //Console.WriteLine("<PrintSpace>");
                        //Console.WriteLine("<TextBlock>");
                        //Console.WriteLine("<TextLine>");

                        tsw.WriteLine("<Page>");
                        tsw.WriteLine("<PrintSpace>");
                        tsw.WriteLine("<TextBlock>");
                        tsw.WriteLine("<TextLine>");

                        PDFStreamEngine engine1 = new PDFStreamEngine();

                        PDStream contents = page.getContents();
                        if (contents != null)
                        {
                            //printer.processStream(page, page.findResources(), page.Contents.Stream);
                           // engine1.processStream(page, page.findResources(), page.getContents().getStream());
                            printer.processStream(page, page.findResources(), page.getContents().getStream());
                           // printer.processTextPosition(page.gettext);

                            
                        }
                        endOfPage(tsw);

                        //Console.WriteLine("</TextLine>");
                        //Console.WriteLine("</TextBlock>");
                        //Console.WriteLine("</PrintSpace>");
                        //Console.WriteLine("</Page>");

                        tsw.WriteLine("</TextLine>");
                        tsw.WriteLine("</TextBlock>");
                        tsw.WriteLine("</PrintSpace>");
                        tsw.WriteLine("</Page>");

                    }
                    tsw.WriteLine("</Layout></alto>");
                }
                finally
                {
                    if (document != null)
                    {
                        document.close();
                    }
                }
            }
        }

        /// <summary>
        /// This will print the documents data.
        /// </summary>
        /// <param name="args"> The command line arguments.
        /// </param>
        /// <exception cref="Exception"> If there is an error parsing the document. </exception>
        //JAVA TO C# CONVERTER WARNING: Method 'throws' clauses are not available in .NET:
        //ORIGINAL LINE: public static void main(String[] args) throws Exception
        [STAThread]
        public static void Main(string[] args)
        {
            PrintWordLocations handler = new PrintWordLocations();
            handler.processDocuments(args);
        }

        /// <summary>
        /// A method provided as an event interface to allow a subclass to perform
        /// some specific functionality when text needs to be processed.
        /// </summary>
        /// <param name="text"> The text to be processed </param>
        protected internal virtual void processTextPosition(TextPosition text)
        {
            char current_character = text.getCharacter().ToLower()[0];

            if (endsWord(current_character))
            {
                emitWordBoxes();
            }
            else
            {
                if (box_list.Count == 0)
                {
                    if (isAlnumOrApostrophe(current_character))
                    {
                        word = word.append(current_character);
                    }

                    box_list.AddFirst(new WordBox(this, text));
                }
                else if (box_list.Last.Value.accepts(text))
                {
                    if (isAlnumOrApostrophe(current_character))
                    {
                        word = word.append(current_character);
                    }

                    box_list.Last.Value.extendBy(text);
                }
                else
                {
                    if (!isHyphen(last_character))
                    {
                        emitWordBoxes();
                    }

                    if (isAlnumOrApostrophe(current_character))
                    {
                        word = word.append(current_character);
                    }

                    box_list.AddFirst(new WordBox(this, text));
                }
            }

            last_character = current_character;
        }

        protected internal virtual void endOfPage(TextWriter tsw)
        {
            //if (box_list.Count > 0)
            //{
            //    emitWordBoxes(tsw);
            //}

            emitWordBoxes(tsw);
        }


        protected internal virtual void emitWordBoxes()
        {
            float pointsToInch1200 = (float)16.6666;
            float mysteryHeightScale = (float)1.5;
            float height;
            float width;
            float hpos;
            float vpos;

            if (word.ToString().Trim().Length > 0)
            {
                foreach (WordBox wordbox in box_list)
                {
                    width = wordbox._width * pointsToInch1200;
                    height = wordbox._height * pointsToInch1200 * mysteryHeightScale;
                    hpos = wordbox._xmin * pointsToInch1200;
                    vpos = wordbox._ymin * pointsToInch1200 - height;

                    Console.WriteLine("<String HEIGHT=\"" + height + "\" WIDTH=\"" + width + "\" HPOS=\"" + hpos + "\" VPOS=\"" + vpos + "\" CONTENT=\"" + word.ToString().Trim() + "\"/>");
                    //tsw.Write("<String HEIGHT=\"" + height + "\" WIDTH=\"" + width + "\" HPOS=\"" + hpos + "\" VPOS=\"" + vpos + "\" CONTENT=\"" + word.ToString().Trim() + "\"/>");
                }
            }

            //word = new StringBuilder("");
            word = new java.lang.StringBuilder("");

            last_character = new char?('\0');
            box_list.Clear();
        }

        protected internal virtual void emitWordBoxes(TextWriter tsw)
        {
            float pointsToInch1200 = (float)16.6666;
            float mysteryHeightScale = (float)1.5;
            float height;
            float width;
            float hpos;
            float vpos;

            if (word.ToString().Trim().Length > 0)
            {
                foreach (WordBox wordbox in box_list)
                {
                    width = wordbox._width * pointsToInch1200;
                    height = wordbox._height * pointsToInch1200 * mysteryHeightScale;
                    hpos = wordbox._xmin * pointsToInch1200;
                    vpos = wordbox._ymin * pointsToInch1200 - height;

                    //Console.WriteLine("<String HEIGHT=\"" + height + "\" WIDTH=\"" + width + "\" HPOS=\"" + hpos + "\" VPOS=\"" + vpos + "\" CONTENT=\"" + word.ToString().Trim() + "\"/>");
                    tsw.Write("<String HEIGHT=\"" + height + "\" WIDTH=\"" + width + "\" HPOS=\"" + hpos + "\" VPOS=\"" + vpos + "\" CONTENT=\"" + word.ToString().Trim() + "\"/>");
                }
            }

            //word = new StringBuilder("");
            word = new java.lang.StringBuilder("");

            last_character = new char?('\0');
            box_list.Clear();
        }

        protected internal virtual bool endsWord(char ch)
        {
            return !(isAlnumOrApostrophe(ch) || isHyphen(ch));
        }

        protected internal virtual bool isAlnumOrApostrophe(char ch)
        {
            return char.IsLetterOrDigit(ch) || (ch == '\'');
        }

        protected internal virtual bool isHyphen(char? ch)
        {
            return ch == '-';
        }

        /// <summary>
        /// This will print the usage for this document.
        /// </summary>
        private static void usage()
        {
            Console.Error.WriteLine("Usage: java org.apache.pdfbox.examples.pdmodel.PrintWordLocations <input-pdf>");
        }
    }

}
